// Jobs manipulation
// Sébastien Gendre <seb@k-7.ch>

#include "jobs.h"
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>


/* Start a given command, return its  */
pid_t job_start_command(int argc, char *argv[]) {
  pid_t pid;
  int exec_status;
  char *redirection_path = NULL;
  int redirection_file;
  int stdout_new_desc = -1;

  pid = fork();

  if (pid < 0) {
    // In case of fork error
    perror("Fork error");
    return -1;
  } else if (pid == 0) {
    // I'm the child process
    // Check if the redirection of stdout to file is requested by user
    redirection_path = job_write_redirection_path_in_argv(argc, argv);
    if (redirection_path) {
      // If yes, open the file path
      redirection_file = open(redirection_path, O_WRONLY | O_CREAT, 0640);
      // Replace stdout file descriptor by a copy of the file
      // descriptor of the openned file
      stdout_new_desc = dup2(redirection_file, STDOUT_FILENO);
      if (stdout_new_desc == -1) {
        perror("Cannot redirect stdout");
      } else {
        // Remove all arguments after the '>'
        job_clean_argv_after_redirection_char(&argc, argv);
      }
    }
    // Exec the command found in PATH env by using execvpe()
    exec_status = execvpe(argv[0], argv, NULL);
    if (exec_status == -1) {
      // In case of error, print the error message
      /* perror(cmd->argv[0]); */
      // Finally, simply print the message requested by the teacher
      printf("Foreground job exited\n");
      exit(EXIT_FAILURE);
    }
  } else {
    // I'm the parent process
    return pid;
  }
}


/* Wait the end of a given command targeted by its pid */
int job_wait_command_pid(pid_t pid) {
    int child_process_status;
    // Wait the child to finish
    waitpid(pid, &child_process_status, 0);
    // Is the child has exited correctly?
    if (WIFEXITED(child_process_status)) {
      // If yes, return its code status
      printf("\nForeground job %d exited with code %d\n", pid, WEXITSTATUS(child_process_status));
      return WEXITSTATUS(child_process_status);
    } else {
      // If not, return -1
      return -1;
    }
}

/* Search file redirection argument ('>') in argv */
int job_write_redirection_position_in_argv(int argc, char *argv[]) {
  // Check each argument in the argv
  for (int i = 0; i < argc; ++i) {
    if (strcmp(argv[i], ">") == 0) {
      // If ">" found, return its position
      return i;
    }
  }
  return -1;
}


/* See if argv has an redirection symbole '>' and return its position */
char* job_write_redirection_path_in_argv(int argc, char *argv[]) {
  int redirect_pos = job_write_redirection_position_in_argv(argc, argv);
  if (redirect_pos == -1) {
    return NULL;
  } else {
    return argv[redirect_pos + 1];
  }
  
}


/* Clean arguments after stdout redirection */
int job_clean_argv_after_redirection_char(int *argc, char **argv) {
  int redirect_arg_pos = job_write_redirection_position_in_argv(*argc, argv);
  // If the redirection char argument position are beyound argv length, return error
  // code
  if (redirect_arg_pos >= *argc) {
    return -1;
  }
  /* // Free all arguments after the redirection argument position */
  /* for (int i = redirect_arg_pos; i < *argc; ++i) { */
  /*   free(argv[i]); */
  /* } */
  // Place a NULL at the redirection argument position
  argv[redirect_arg_pos] = NULL;
  // Define the new argc value
  *argc = redirect_arg_pos;
  // Reduce argv length
  argv = realloc(argv, (*argc+1)*sizeof(char*));
  return 0;
}


// Execute a given command as job
int job_exec_command(int argc, char *argv[]) {
  // Fork of the given command
  pid_t pid;

  // Start the command
  pid = job_start_command(argc, argv);
  // Wait until the command stop
  return job_wait_command_pid(pid);
 }

